/*********************************************\
*   Tektronix 4957 graphic tablet driver      *
*                                             *
\*********************************************/

#include <InterfaceDefs.h>
#include <SerialPort.h>
#include <add-ons/input_server/InputServerDevice.h>
#include <List.h>
#include <OS.h>
#include <SupportDefs.h>

extern "C" _EXPORT BInputServerDevice* instantiate_input_device();
 
const int MAXCO=5944;

//##################################################################

class TabletDevice : public BInputServerDevice
{
public:
						TabletDevice();
	virtual				~TabletDevice();
	virtual status_t	InitCheck();
	virtual status_t	Start(const char *device, void *cookie);
	virtual	status_t	Stop (const char *device, void *cookie);
	virtual status_t	Control(const char	*device,
								void		*cookie,
								uint32		code, 
								BMessage	*message);
	int32 Thread();
	BSerialPort SP;
	bool StopThread;
	thread_id TID;
	int LastX,LastY,LastB;
};
//##################################################################

BInputServerDevice* instantiate_input_device()
{
	return (new TabletDevice());
}

//------------------------------------------------------------------
TabletDevice::TabletDevice():BInputServerDevice()
{
 input_device_ref TekDevice={"Tektronix 4957",B_POINTING_DEVICE,NULL};
 input_device_ref *DeviceList[2]={&TekDevice,NULL};
 
 RegisterDevices(DeviceList);
}

//------------------------------------------------------------------
TabletDevice::~TabletDevice()
{
}

//------------------------------------------------------------------
status_t TabletDevice::InitCheck()
{
 if ( BInputServerDevice::InitCheck() != B_OK ) return B_ERROR;

 SP.SetDataRate(B_9600_BPS);
 SP.SetDataBits(B_DATA_BITS_7);
 SP.SetStopBits(B_STOP_BITS_1);
 SP.SetParityMode(B_ODD_PARITY);
 SP.SetFlowControl(0);
 SP.SetBlocking(true);
 SP.SetTimeout(200000);

 char devName[B_OS_NAME_LENGTH];
 int32 I;
 for (I=SP.CountDevices()-1;I>=0;I--) {
	SP.GetDeviceName(I,devName);
	if (SP.Open(devName)>0) {

		// Port opened ... configure tablet...
		
		SP.Write("\x1B" "Z",2); // ESC Z : Reset
		snooze(100000);
		SP.ClearInput();
		SP.Write("\x1B" "x",2); // ESC x : Shall return .#EAFF
		
		ssize_t Lo=SP.WaitForInput();
		if (Lo<2) { SP.Close(); continue; } // Wrong port ???
		char Buf[10];
		SP.Read(Buf,2);
		if ((Buf[0]!='.')||(Buf[1]!='#')) { SP.Close(); continue; } // Bad answer 
		SP.Write("\x1B" "C5",3); // ESC C 5 : 0.05 mm steps : max=5944
		SP.Write("\x1B" "I001",5); // ESC I 0 0 1 : Emit data when movment
		SP.Write("\x1B" "M0",3); // Emit when diffrerent
		SP.Write("\x1B" "R6",3); // ESC R 6 : Maximum speed
		SP.ClearInput(); // Flush garbage
		
		LastX=0;
		LastY=0;
		LastB=0;
		return B_OK;
		}
	}
 return B_ERROR;
}

//------------------------------------------------------------------
int32 MThread(void *T)
{
 return ((TabletDevice *)T)->Thread();
}

//------------------------------------------------------------------
status_t TabletDevice::Start(const char *device, void *cookie)
{
 StopThread = false;
 TID=spawn_thread((thread_func)(MThread),device, B_DISPLAY_PRIORITY,this);
 resume_thread(TID);
 return B_NO_ERROR;
}

//------------------------------------------------------------------
status_t TabletDevice::Stop(const char *device, void *cookie)
{
 status_t exit;
 StopThread = true;
 wait_for_thread( TID, &exit );
 return B_NO_ERROR;
}

//------------------------------------------------------------------
status_t TabletDevice::Control(const char *device, void *cookie,
							   uint32 code, BMessage *message )
{
 return B_NO_ERROR;
}


//------------------------------------------------------------------
int32 TabletDevice::Thread()
{
 char Buf[10];
 do {
	if (StopThread) break;
	ssize_t Lo=SP.WaitForInput();
	if (Lo<8) continue;
	SP.Read(&Buf[0],1);
	if ((Buf[0]&0x40)==0) continue;
	SP.Read(&Buf[1],7);
	int X,Y,B;
	X=(Buf[2]&0x3F)|((Buf[3]&0x3F)<<6) | ((Buf[4]&0x3F)<<12);
	Y=(Buf[5]&0x3F)|((Buf[6]&0x3F)<<6) | ((Buf[7]&0x3F)<<12);
	Y=MAXCO-Y;
	B=Buf[1]&7;
	
	BMessage* Msg;
	if (B==LastB) Msg=new BMessage(B_MOUSE_MOVED);
	else if (B>LastB) Msg=new BMessage(B_MOUSE_DOWN);
	else Msg=new BMessage(B_MOUSE_UP);
	
	bigtime_t When = system_time();
	Msg->AddInt64( "when",When);
	Msg->AddFloat("x",((float)X/MAXCO));
	Msg->AddFloat("y",((float)Y/MAXCO));
	Msg->AddFloat("be:tablet_x",((float)X/MAXCO));
	Msg->AddFloat("be_tablet_y",((float)Y/MAXCO));
	
	Msg->AddInt32("buttons",B);
	LastB=B;
	LastX=X;
	LastY=Y;
	
	EnqueueMessage(Msg);
	} while (true);
}

